@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,14 +31,14 @@ from renpy.display.layout import Container
 import renpy.display.accelerator
 
 # The null object that's used if we don't have a defined child.
-null = None 
+null = None
 
 def get_null():
     global null
-    
+
     if null is None:
         null = renpy.display.layout.Null()
-        
+
     return null
 
 # Convert a position from cartesian to polar coordinates.
@@ -55,22 +55,22 @@ def cartesian_to_polar(x, y, xaround, yaround):
 
     if angle < 0:
         angle += 360
-    
+
     return angle, radius
-    
+
 def polar_to_cartesian(angle, radius, xaround, yaround):
     """
     Converts polart coordinates to cartesian coordinates.
     """
-    
+
     angle = angle * math.pi / 180
-    
+
     dx = radius * math.sin(angle)
     dy = -radius * math.cos(angle)
-    
+
     x = type(xaround)(xaround + dx)
     y = type(yaround)(yaround + dy)
-    
+
     return x, y
 
 def first_not_none(*args):
@@ -96,7 +96,8 @@ class TransformState(renpy.object.Object):
     default_yoffset = None
     transform_anchor = False
     additive = 0.0
-    
+    debug = None
+
     def __init__(self): # W0231
         self.alpha = 1
         self.additive = 0.0
@@ -106,14 +107,14 @@ class TransformState(renpy.object.Object):
         self.zoom = 1
         self.xzoom = 1
         self.yzoom = 1
-        
+
         self.xpos = None
         self.ypos = None
         self.xanchor = None
         self.yanchor = None
         self.xoffset = 0
         self.yoffset = 0
-        
+
         self.xaround = 0.0
         self.yaround = 0.0
         self.xanchoraround = 0.0
@@ -128,19 +129,21 @@ class TransformState(renpy.object.Object):
 
         self.delay = 0
 
+        self.debug = None
+
         # Note: When adding a new property, we need to add it to:
         # - take_state
         # - diff
         # - renpy.atl.PROPERTIES
         # - Proxies in Transform
-        
+
         # Default values for various properties, taken from our
         # parent.
         self.default_xpos = None
         self.default_ypos = None
         self.default_xanchor = None
         self.default_yanchor = None
-        
+
     def take_state(self, ts):
 
         self.alpha = ts.alpha
@@ -151,7 +154,7 @@ class TransformState(renpy.object.Object):
         self.zoom = ts.zoom
         self.xzoom = ts.xzoom
         self.yzoom = ts.yzoom
-        
+
         self.xaround = ts.xaround
         self.yaround = ts.yaround
         self.xanchoraround = ts.xanchoraround
@@ -164,16 +167,18 @@ class TransformState(renpy.object.Object):
         self.corner2 = ts.corner2
         self.size = ts.size
 
+        self.debug = ts.debug
+
         # Take the computed position properties, not the
         # raw ones.
         (self.default_xpos,
-         self.default_ypos,         
+         self.default_ypos,
          self.default_xanchor,
          self.default_yanchor,
          self.xoffset,
          self.yoffset,
          self.subpixel) = ts.get_placement()
-        
+
     # Returns a dict, with p -> (old, new) where p is a property that
     # has changed between this object and the new object.
     def diff(self, newts):
@@ -189,12 +194,12 @@ class TransformState(renpy.object.Object):
                 new_value = default_new
             else:
                 new_value = new
-                
+
             if old is None:
                 old_value = default_old
             else:
                 old_value = old
-                
+
             if new_value != old_value:
                 rv[prop] = (old_value, new_value)
 
@@ -206,7 +211,7 @@ class TransformState(renpy.object.Object):
         diff2("zoom", newts.zoom, self.zoom)
         diff2("xzoom", newts.xzoom, self.xzoom)
         diff2("yzoom", newts.yzoom, self.yzoom)
-        
+
         diff2("xaround", newts.xaround, self.xaround)
         diff2("yaround", newts.yaround, self.yaround)
         diff2("xanchoraround", newts.xanchoraround, self.xanchoraround)
@@ -227,7 +232,7 @@ class TransformState(renpy.object.Object):
         diff4("ypos", newts.ypos, newts.default_ypos, self.ypos, self.default_ypos)
         diff4("yanchor", newts.yanchor, newts.default_yanchor, self.yanchor, self.default_yanchor)
         diff2("yoffset", newts.yoffset, self.yoffset)
-        
+
         return rv
 
     def get_placement(self, cxoffset=0, cyoffset=0):
@@ -241,10 +246,10 @@ class TransformState(renpy.object.Object):
             self.yoffset + cyoffset,
             self.subpixel,
             )
-                    
+
     # These update various properties.
     def get_xalign(self):
-        return self.xpos 
+        return self.xpos
 
     def set_xalign(self, v):
         self.xpos = v
@@ -253,7 +258,7 @@ class TransformState(renpy.object.Object):
     xalign = property(get_xalign, set_xalign)
 
     def get_yalign(self):
-        return self.ypos 
+        return self.ypos
 
     def set_yalign(self, v):
         self.ypos = v
@@ -263,7 +268,7 @@ class TransformState(renpy.object.Object):
 
     def get_around(self):
         return (self.xaround, self.yaround)
-    
+
     def set_around(self, value):
         self.xaround, self.yaround = value
         self.xanchoraround, self.yanchoraround = None, None
@@ -271,10 +276,10 @@ class TransformState(renpy.object.Object):
     def set_alignaround(self, value):
         self.xaround, self.yaround = value
         self.xanchoraround, self.yanchoraround = value
-        
+
     around = property(get_around, set_around)
     alignaround = property(get_around, set_alignaround)
-        
+
     def get_angle(self):
         xpos = first_not_none(self.xpos, self.default_xpos, 0)
         ypos = first_not_none(self.ypos, self.default_ypos, 0)
@@ -296,7 +301,7 @@ class TransformState(renpy.object.Object):
 
         if self.xanchoraround:
             self.xanchor, self.yanchor = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
-        
+
     def set_radius(self, value):
         xpos = first_not_none(self.xpos, self.default_xpos, 0)
         ypos = first_not_none(self.ypos, self.default_ypos, 0)
@@ -306,7 +311,7 @@ class TransformState(renpy.object.Object):
 
         if self.xanchoraround:
             self.xanchor, self.yanchor = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
-        
+
     angle = property(get_angle, set_angle)
     radius = property(get_radius, set_radius)
 
@@ -359,7 +364,7 @@ class TransformState(renpy.object.Object):
 
     xcenter = property(get_xcenter, set_xcenter)
     ycenter = property(get_ycenter, set_ycenter)
-    
+
 class Proxy(object):
     """
     This class proxies a field from the transform to its state.
@@ -373,7 +378,7 @@ class Proxy(object):
 
     def __set__(self, instance, value):
         return setattr(instance.state, self.name, value)
-    
+
 class Transform(Container):
     """
     Documented in sphinx, because we can't scan this object.
@@ -381,7 +386,7 @@ class Transform(Container):
 
     __version__ = 5
     transform_event_responder = True
-    
+
     # Proxying things over to our state.
     alpha = Proxy("alpha")
     additive = Proxy("additive")
@@ -396,7 +401,7 @@ class Transform(Container):
     ypos = Proxy("ypos")
     xanchor = Proxy("xanchor")
     yanchor = Proxy("yanchor")
-    
+
     xalign = Proxy("xalign")
     yalign = Proxy("yalign")
 
@@ -413,7 +418,7 @@ class Transform(Container):
     pos = Proxy("pos")
     anchor = Proxy("anchor")
     align = Proxy("align")
-    
+
     crop = Proxy("crop")
     corner1 = Proxy("corner1")
     corner2 = Proxy("corner2")
@@ -429,7 +434,9 @@ class Transform(Container):
 
     xcenter = Proxy("xcenter")
     ycenter = Proxy("ycenter")
-    
+
+    debug = Proxy("debug")
+
     def after_upgrade(self, version):
 
         if version < 1:
@@ -464,7 +471,7 @@ class Transform(Container):
         if version < 5:
             self.replaced_request = False
             self.replaced_response = True
-            
+
     DEFAULT_ARGUMENTS = {
             "selected_activate" : { },
             "selected_hover" : { },
@@ -481,7 +488,10 @@ class Transform(Container):
     active = False
     children = False
     arguments = DEFAULT_ARGUMENTS
-    
+
+    # Default before we set this.
+    child_size = (0, 0)
+
     def __init__(self,
                  child=None,
                  function=None,
@@ -489,12 +499,12 @@ class Transform(Container):
                  style='transform',
                  focus=None,
                  default=False,
-                
+
                  **kwargs):
 
         self.kwargs = kwargs
         self.style_arg = style
-        
+
         super(Transform, self).__init__(style=style, focus=focus, default=default)
 
         self.function = function
@@ -506,7 +516,7 @@ class Transform(Container):
         self.state = TransformState()
 
         self.arguments = dict((k, {}) for k in self.DEFAULT_ARGUMENTS)
-        
+
         # Split up the keyword arguments.
         for k, v in kwargs.iteritems():
             if "_" in k:
@@ -520,10 +530,10 @@ class Transform(Container):
 
             if prop not in renpy.atl.PROPERTIES:
                 raise Exception("Unknown transform property: %r" % prop)
-            
+
             self.arguments[prefix][prop] = v
 
-            
+
         # Apply the keyword arguments.
         for k, v in kwargs.iteritems():
             setattr(self.state, k, v)
@@ -536,7 +546,7 @@ class Transform(Container):
 
         # Have we been requested to hide?
         self.hide_request = False
-        
+
         # True if it's okay for us to hide.
         self.hide_response = True
 
@@ -545,7 +555,7 @@ class Transform(Container):
 
         # True if it's okay for us to replaced.
         self.replaced_response = True
-        
+
         self.st = 0
         self.at = 0
         self.st_offset = 0
@@ -556,9 +566,9 @@ class Transform(Container):
     def visit(self):
         if self.child is None:
             return [ ]
-        else:        
+        else:
             return [ self.child ]
-    
+
     # The default function chooses entries from self.arguments that match
     # the style prefix, and applies them to the state.
     def default_function(self, state, st, at):
@@ -575,27 +585,27 @@ class Transform(Container):
         for i in prefixes:
             for k, v in self.arguments[i].iteritems():
                 setattr(state, k, v)
-            
+
         return None
-        
+
     def set_transform_event(self, event):
         if self.child is not None:
             self.child.set_transform_event(event)
-        
+
         super(Transform, self).set_transform_event(event)
-        
-        
+
+
     def take_state(self, t):
         """
         Takes the transformation state from object t into this object.
         """
-        
+
         self.state.take_state(t.state)
 
         # The arguments will be applied when the default function is
         # called.
 
-            
+
     def take_execution_state(self, t):
         """
         Takes the execution state from object t into this object. This is
@@ -610,6 +620,8 @@ class Transform(Container):
         self.state.xanchor = t.state.xanchor
         self.state.yanchor = t.state.yanchor
 
+        self.child_st_base = t.child_st_base
+
         if isinstance(self.child, Transform) and isinstance(t.child, Transform):
             self.child.take_execution_state(t.child)
 
@@ -630,12 +642,12 @@ class Transform(Container):
 
     def _change_transform_child(self, child):
         rv = self.copy()
-        
+
         if self.child is not None:
             rv.set_child(self.child._change_transform_child(child))
-        
+
         return rv
-    
+
     def _hide(self, st, at, kind):
 
         if not self.child:
@@ -659,37 +671,37 @@ class Transform(Container):
 
         if d.function is not None:
             d.function(d, st + d.st_offset, at + d.at_offset)
-       
+
         new_child = d.child._hide(st, at, kind)
 
         if new_child is not None:
             d.child = new_child
             d.hide_response = False
             d.replaced_response = False
-                
+
         if (not d.hide_response) or (not d.replaced_response):
             renpy.display.render.redraw(d, 0)
             return d
-        
+
         return None
-        
+
     def set_child(self, child):
-        
+
         child = renpy.easy.displayable(child)
-        
+
         self.child = child
         self.child_st_base = self.st
 
         child.per_interact()
 
         renpy.display.render.redraw(self, 0)
-        
+
     def update_state(self):
         """
         This updates the state to that at self.st, self.at.
         """
 
-        # If we have to, call the function that updates this transform.        
+        # If we have to, call the function that updates this transform.
         if self.function is not None:
             fr = self.function(self, self.st, self.at)
         else:
@@ -700,7 +712,7 @@ class Transform(Container):
             renpy.display.render.redraw(self, fr)
 
         state = self.state
-        
+
         self.active = True
 
         # Use non-None elements of the child placement as defaults.
@@ -718,21 +730,21 @@ class Transform(Container):
             if pos[3] is not None:
                 state.default_yanchor = pos[3]
 
-            state.subpixel |= pos[6]   
-        
+            state.subpixel |= pos[6]
+
     # The render method is now defined in accelerator.pyx.
-            
+
     def event(self, ev, x, y, st):
 
         if self.hide_request:
             return None
-        
+
         children = self.children
         offsets = self.offsets
-        
+
         if not offsets:
             return None
-        
+
         for i in xrange(len(self.children)-1, -1, -1):
 
             d = children[i]
@@ -744,36 +756,36 @@ class Transform(Container):
             # Transform screen coordinates to child coordinates.
             cx, cy = self.forward.transform(cx, cy)
 
-            rv = d.event(ev, cx, cy, st)    
+            rv = d.event(ev, cx, cy, st)
             if rv is not None:
                 return rv
-                
+
         return None
-    
+
     def __call__(self, child=None, take_state=True):
 
         if child is None:
             child = self.child
 
-        # If we don't have a child for some reason, set it to null.            
+        # If we don't have a child for some reason, set it to null.
         if child is None:
             child = get_null()
-        
+
         rv = Transform(
             child=child,
             function=self.function,
             style=self.style_arg,
             **self.kwargs)
-        
+
         rv.take_state(self)
 
         return rv
-    
+
     def get_placement(self):
 
         if not self.active:
             self.update_state()
-            
+
         if self.child is not None:
             _cxpos, _cypos, _cxanchor, _cyanchor, cxoffset, cyoffset, _csubpixel = self.child.get_placement()
         else:
@@ -781,36 +793,36 @@ class Transform(Container):
             cyoffset = 0
 
         cxoffset = cxoffset or 0
-        cyoffset = cyoffset or 0 
-        
+        cyoffset = cyoffset or 0
+
         rv = self.state.get_placement(cxoffset, cyoffset)
 
         if self.state.transform_anchor:
 
             xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel = rv
             if (xanchor is not None) and (yanchor is not None):
-    
+
                 cw, ch = self.child_size
                 rw, rh = self.render_size
-                
+
                 if isinstance(xanchor, float):
                     xanchor *= cw
                 if isinstance(yanchor, float):
                     yanchor *= ch
-                
+
                 xanchor -= cw / 2.0
                 yanchor -= ch / 2.0
-            
+
                 xanchor, yanchor = self.reverse.transform(xanchor, yanchor)
-    
+
                 xanchor += rw / 2.0
                 yanchor += rh / 2.0
-    
+
                 xanchor = renpy.display.core.absolute(xanchor)
                 yanchor = renpy.display.core.absolute(yanchor)
-            
+
                 rv = (xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel)
-        
+
         return rv
 
     def update(self):
@@ -818,7 +830,7 @@ class Transform(Container):
         This should be called when a transform property field is updated outside
         of the callback method, to ensure that the change takes effect.
         """
-        
+
         renpy.display.render.invalidate(self)
 
     def parameterize(self, name, parameters):
@@ -828,25 +840,25 @@ class Transform(Container):
 
         # Note the call here.
         return self()
-    
+
     def _show(self):
         self.update_state()
-        
+
 Transform.render = types.MethodType(renpy.display.accelerator.transform_render, None, Transform)
 
 class ATLTransform(renpy.atl.ATLTransformBase, Transform):
-    
+
     def __init__(self, atl, child=None, context={}, parameters=None, **properties):
         renpy.atl.ATLTransformBase.__init__(self, atl, context, parameters)
         Transform.__init__(self, child=child, function=self.execute, **properties)
 
         self.raw_child = self.child
-        
+
     def _show(self):
         super(ATLTransform, self)._show()
         self.execute(self, self.st, self.at)
-    
-    
+
+
 class Motion(Container):
     """
     This is used to move a child displayable around the screen. It
@@ -903,7 +915,7 @@ class Motion(Container):
 
         if child is None:
             child = new_widget
-            
+
         if delay is None and not repeat:
             delay = period
 
@@ -922,7 +934,7 @@ class Motion(Container):
         self.add_sizes = add_sizes
 
         self.position = None
-        
+
 
     def get_placement(self):
 
@@ -930,7 +942,7 @@ class Motion(Container):
             return super(Motion, self).get_placement()
         else:
             return self.position + (self.style.xoffset, self.style.yoffset, self.style.subpixel)
-                
+
     def render(self, width, height, st, at):
 
         if self.anim_timebase:
@@ -944,9 +956,9 @@ class Motion(Container):
                 if self.repeat:
                     t = t % self.period
             else:
-                t = self.period                    
+                t = self.period
         elif self.delay and t >= self.delay:
-            t = self.delay            
+            t = self.delay
             if self.repeat:
                 t = t % self.period
         elif self.repeat:
@@ -980,7 +992,7 @@ class Motion(Container):
             res = self.function(t)
 
         res = tuple(res)
-            
+
         if len(res) == 2:
             self.position = res + (self.style.xanchor, self.style.yanchor)
         else:
@@ -993,7 +1005,7 @@ class Motion(Container):
 
         return rv
 
-        
+
 class Interpolate(object):
 
     anchors = {
@@ -1021,11 +1033,11 @@ class Interpolate(object):
                     a = a * c
                 if type(b) is float:
                     b = b * c
-                            
+
             rv = a + t * (b - a)
-            
+
             return renpy.display.core.absolute(rv)
-            
+
         return [ interp(a, b, c) for a, b, c in zip(self.start, self.end, sizes) ]
 
 
@@ -1043,7 +1055,7 @@ def Pan(startpos, endpos, time, child=None, repeat=False, bounce=False,
 
     @param endpos: The coordinates of the upper-left corner of the
     screen, relative to the image, after time has elapsed.
-    
+
     @param time: The time it takes to pan from startpos to endpos.
 
     @param child: The child displayable.
@@ -1066,11 +1078,11 @@ def Pan(startpos, endpos, time, child=None, repeat=False, bounce=False,
 
     x0, y0 = startpos
     x1, y1 = endpos
-    
+
     return Motion(Interpolate((-x0, -y0), (-x1, -y1)),
                   time,
                   child,
-                  repeat=repeat, 
+                  repeat=repeat,
                   bounce=bounce,
                   style=style,
                   anim_timebase=anim_timebase,
@@ -1083,14 +1095,14 @@ def Move(startpos, endpos, time, child=None, repeat=False, bounce=False,
     """
     This is used to pan over a child displayable relative to
     the containing area. It works by interpolating the placement of the
-    the child, over time. 
+    the child, over time.
 
     @param startpos: The initial coordinates of the child
     relative to the containing area.
 
     @param endpos: The coordinates of the child at the end of the
     move.
-    
+
     @param time: The time it takes to move from startpos to endpos.
 
     @param child: The child displayable.
@@ -1114,7 +1126,7 @@ def Move(startpos, endpos, time, child=None, repeat=False, bounce=False,
     return Motion(Interpolate(startpos, endpos),
                   time,
                   child,
-                  repeat=repeat, 
+                  repeat=repeat,
                   bounce=bounce,
                   anim_timebase=anim_timebase,
                   style=style,
@@ -1132,7 +1144,7 @@ class Revolver(object):
         self.cor = cor
         self.pos = pos
         self.child = child
-        
+
     def __call__(self, t, (w, h, cw, ch)):
 
         # Converts a float to an integer in the given range, passes
@@ -1145,12 +1157,12 @@ class Revolver(object):
                 return int(x * r)
             else:
                 return x
-        
+
         if self.pos is None:
             pos = self.child.get_placement()
         else:
             pos = self.pos
-            
+
         xpos, ypos, xanchor, yanchor, _xoffset, _yoffset, _subpixel = pos
 
         xpos = fti(xpos, w)
@@ -1170,7 +1182,7 @@ class Revolver(object):
 
         angle = self.start + (self.end - self.start) * t
         angle *= math.pi / 180
-        
+
         # The center of rotation, relative to the xaround.
         x = xpos - xanchor + xcor - xaround
         y = ypos - yanchor + ycor - yaround
@@ -1200,30 +1212,30 @@ def zoom_render(crend, x, y, w, h, zw, zh, bilinear):
     """
     This creates a render that zooms its child.
 
-    `crend` - The render of the child. 
+    `crend` - The render of the child.
     `x`, `y`, `w`, `h` - A rectangle inside the child.
     `zw`, `zh` - The size the rectangle is rendered to.
     `bilinear` - Should we be rendering in bilinear mode?
     """
-    
+
     rv = renpy.display.render.Render(zw, zh)
 
     if zw == 0 or zh == 0 or w == 0 or h == 0:
         return rv
 
-    
+
     rv.forward = renpy.display.render.Matrix2D(w / zw, 0, 0, h / zh)
     rv.reverse = renpy.display.render.Matrix2D(zw / w, 0, 0, zh / h)
 
     rv.clipping = True
-    
+
     rv.blit(crend, rv.reverse.transform(-x, -y))
 
     return rv
 
 
 class ZoomCommon(renpy.display.core.Displayable):
-    def __init__(self, 
+    def __init__(self,
                  time, child,
                  end_identity=False,
                  after_child=None,
@@ -1258,7 +1270,7 @@ class ZoomCommon(renpy.display.core.Displayable):
         self.time = time
         self.child = child
         self.repeat = repeat
-        
+
         if after_child:
             self.after_child = renpy.easy.displayable(after_child)
         else:
@@ -1266,12 +1278,12 @@ class ZoomCommon(renpy.display.core.Displayable):
                 self.after_child = child
             else:
                 self.after_child = None
-        
+
         self.time_warp = time_warp
         self.bilinear = bilinear
         self.opaque = opaque
         self.anim_timebase = anim_timebase
-        
+
 
     def visit(self):
         return [ self.child, self.after_child ]
@@ -1282,7 +1294,7 @@ class ZoomCommon(renpy.display.core.Displayable):
             t = at
         else:
             t = st
-        
+
         if self.time:
             done = min(t / self.time, 1.0)
         else:
@@ -1293,7 +1305,7 @@ class ZoomCommon(renpy.display.core.Displayable):
 
         if renpy.game.less_updates:
             done = 1.0
-            
+
         self.done = done
 
         if self.after_child and done == 1.0:
@@ -1305,12 +1317,12 @@ class ZoomCommon(renpy.display.core.Displayable):
         rend = renpy.display.render.render(self.child, width, height, st, at)
 
         rx, ry, rw, rh, zw, zh = self.zoom_rectangle(done, rend.width, rend.height)
-                
+
         if rx < 0 or ry < 0 or rx + rw > rend.width or ry + rh > rend.height:
             raise Exception("Zoom rectangle %r falls outside of %dx%d parent surface." % ((rx, ry, rw, rh), rend.width, rend.height))
 
         rv = zoom_render(rend, rx, ry, rw, rh, zw, zh, self.bilinear)
-            
+
         if self.done < 1.0:
             renpy.display.render.redraw(self, 0)
 
@@ -1322,7 +1334,7 @@ class ZoomCommon(renpy.display.core.Displayable):
             done = 1.0
         else:
             done = min(st / self.time, 1.0)
-            
+
         if done == 1.0 and self.after_child:
             return self.after_child.event(ev, x, y, st)
         else:
@@ -1385,14 +1397,14 @@ class SizeZoom(ZoomCommon):
 
         zw = sw + (ew - sw) * done
         zh = sh + (eh - sh) * done
-        
+
         return 0, 0, width, height, zw, zh
 
 
 class RotoZoom(renpy.display.core.Displayable):
 
     transform = None
-    
+
     def __init__(self,
                  rot_start,
                  rot_end,
@@ -1424,13 +1436,13 @@ class RotoZoom(renpy.display.core.Displayable):
         self.zoom_delay = zoom_delay
 
         self.child = renpy.easy.displayable(child)
-        
+
         self.rot_repeat = rot_repeat
         self.zoom_repeat = zoom_repeat
 
         self.rot_bounce = rot_bounce
         self.zoom_bounce = zoom_bounce
-        
+
         self.rot_anim_timebase = rot_anim_timebase
         self.zoom_anim_timebase = zoom_anim_timebase
 
@@ -1439,11 +1451,11 @@ class RotoZoom(renpy.display.core.Displayable):
 
         self.opaque = opaque
 
-        
+
     def visit(self):
         return [ self.child ]
 
-    
+
     def render(self, width, height, st, at):
 
         if self.rot_anim_timebase:
@@ -1483,31 +1495,31 @@ class RotoZoom(renpy.display.core.Displayable):
         if renpy.game.less_updates:
             rot_time = 1.0
             zoom_time = 1.0
-            
+
         rot_time = min(rot_time, 1.0)
         zoom_time = min(zoom_time, 1.0)
-        
+
         if self.rot_time_warp:
             rot_time = self.rot_time_warp(rot_time)
 
         if self.zoom_time_warp:
             zoom_time = self.zoom_time_warp(zoom_time)
 
-            
+
         angle = self.rot_start + (1.0 * self.rot_end - self.rot_start) * rot_time
         zoom = self.zoom_start + (1.0 * self.zoom_end - self.zoom_start) * zoom_time
         # angle = -angle * math.pi / 180
 
-        zoom = max(zoom, 0.001) 
-        
+        zoom = max(zoom, 0.001)
+
         if self.transform is None:
             self.transform = Transform(self.child)
-            
+
         self.transform.rotate = angle
         self.transform.zoom = zoom
 
         rv = renpy.display.render.render(self.transform, width, height, st, at)
-        
+
         if rot_time <= 1.0 or zoom_time <= 1.0:
             renpy.display.render.redraw(self.transform, 0)
 
